﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;

namespace TEC_Print_RS232C
{
    public partial class Form1 : Form
    {
        public delegate void myDelegate();
        public string inputData, recvData;
       
        public Form1()
        {
            InitializeComponent();
        }

        private void Port_Open()
        {
            serialPort1.PortName = comboBox1.GetItemText(comboBox1.SelectedItem);
            serialPort1.BaudRate = 9600;
            serialPort1.Parity = System.IO.Ports.Parity.None;
            serialPort1.DataBits = 8;
            serialPort1.StopBits = System.IO.Ports.StopBits.One;
            serialPort1.DtrEnable = false;
            //serialPort1.RtsEnable = false;
            //serialPort1.Handshake = System.IO.Ports.Handshake.RequestToSendXOnXOff;
            serialPort1.RtsEnable = true;  //B-FV4 사용시
            serialPort1.Handshake = System.IO.Ports.Handshake.None; //B-FV4사용시  

            serialPort1.Open();
        }

        private int WriteCommASCII(String st) //문자열 출력하는 함수 (ASCII Code로 전송)
        {
            try
            {
                if (serialPort1.IsOpen)
                {
                    Encoding oEnc = ASCIIEncoding.GetEncoding(949);
                    byte[] tt = oEnc.GetBytes(st);
                    serialPort1.Write(tt, 0, tt.Length);
                    return 1;
                }
                else return -1;
            }
            catch (IOException e)
            {
                MessageBox.Show(e.ToString());
                return 0;
            }
        }

        private int WriteCommUTF8(String st) //문자열 출력하는 함수 (UTF-8 Code로 전송)
        {
            try
            {
                if (serialPort1.IsOpen)
                {

                    Encoding oEnc = UTF8Encoding.GetEncoding("utf-8");
                    byte[] tt = oEnc.GetBytes(st);
                    serialPort1.Write(tt, 0, tt.Length);
                    return 1;
                }
                else return -1;
            }
            catch (IOException e)
            {
                MessageBox.Show(e.ToString());
                return 0;
            }
        }

        //이미지 처리 한글 인쇄 (x위치, y위치, 폰트명, 폰트사이즈, bold 유무, 회전(0도=>0, 90도=>1, 180도=>2, 270도=>3), 데이터)
        public void PutHangul(int x, int y, string fname, int fsize, bool fbold, int rot, string data)
        {
            int aHeight;
            int aWidth;
            int i, j, k;
            Byte bb;
            string st;
            Font fnt;

            

            using (ImageForm img_frm = new ImageForm())
            {
                Graphics g = img_frm.CreateGraphics();
                if (fbold == false)
                {
                    fnt = new Font(fname, fsize, FontStyle.Regular, GraphicsUnit.Pixel);
                }
                else
                {
                    fnt = new Font(fname, fsize, FontStyle.Bold, GraphicsUnit.Pixel);
                }

                SizeF s = g.MeasureString(data, fnt);
                Bitmap bmp = new Bitmap((int)s.Width, (int)s.Height);
                g = Graphics.FromImage(bmp);
                g.Clear(Color.White);
                if ((rot == 0) || (rot == 2))
                {
                    aHeight = (int)s.Height;
                    aWidth = (((int)s.Width + 7) >> 3) * 8;
                    st = "{" + "SG;" + String.Format("{0:0000}", x) + "," + String.Format("{0:0000}", y) + "," + String.Format("{0:0000}", aWidth) + "," + String.Format("{0:0000}", aHeight) + ",0,";
                    WriteCommASCII(st);
                }
                else
                {
                    aHeight = (((int)s.Height + 7) >> 3) * 8;
                    aWidth = (int)s.Width;
                    st = "{" + "SG;" + String.Format("{0:0000}", x) + "," + String.Format("{0:0000}", y) + "," + String.Format("{0:0000}", aHeight) + "," + String.Format("{0:0000}", aWidth) + ",0,";
                    WriteCommASCII(st);
                }
                g.DrawString(data, fnt, Brushes.Black, 0, 0);

                if (rot == 0)
                {
                    j = 0;
                    while (j < aHeight)
                    {
                        i = 0;
                        while (i < aWidth)
                        {
                            bb = 0;
                            for (k = 0; k < 4; k++)
                            {
                                if ((i + k) < bmp.Width)
                                {
                                    if (bmp.GetPixel(i + k, j) == Color.FromArgb(255, 0, 0, 0))
                                    {
                                        bb = (byte)(bb + (1 << (3 - k)));
                                    }
                                }
                            }
                            bb += 48;
                            st = Convert.ToString((char)(bb));
                            WriteCommASCII(st); ;
                            i += 4;
                        }
                        j++;
                    }
                }

                if (rot == 2)
                {
                    j = aHeight - 1;
                    while (j >= 0)
                    {
                        i = aWidth - 1;
                        while (i >= 0)
                        {
                            bb = 0;
                            for (k = 0; k < 4; k++)
                            {
                                if ((i - k) < bmp.Width)
                                {
                                    if (bmp.GetPixel(i - k, j) == Color.FromArgb(255, 0, 0, 0))
                                    {
                                        bb = (byte)(bb + (1 << (3 - k)));
                                    }
                                }
                            }
                            bb += 48;
                            st = Convert.ToString((char)(bb));
                            WriteCommASCII(st);
                            i -= 4;
                        }
                        j--;
                    }
                }

                if (rot == 1)
                {
                    j = 0;
                    while (j < aWidth)
                    {
                        i = aHeight - 1;
                        while (i >= 0)
                        {
                            bb = 0;
                            for (k = 0; k < 4; k++)
                            {
                                if ((i - k) < bmp.Height)
                                {
                                    if (bmp.GetPixel(j, i - k) == Color.FromArgb(255, 0, 0, 0))
                                    {
                                        bb = (byte)(bb + (1 << (3 - k)));
                                    }
                                }
                            }
                            bb += 48;
                            st = Convert.ToString((char)(bb));
                            WriteCommASCII(st);
                            i -= 4;
                        }
                        j++;
                    }
                }

                if (rot == 3)
                {
                    j = aWidth - 1;
                    while (j >= 0)
                    {
                        i = 0;
                        while (i < aHeight)
                        {
                            bb = 0;
                            for (k = 0; k < 4; k++)
                            {
                                if ((i + k) < bmp.Height)
                                {
                                    if (bmp.GetPixel(j, i + k) == Color.FromArgb(255, 0, 0, 0))
                                    {
                                        bb = (byte)(bb + (1 << (3 - k)));
                                    }
                                }
                            }
                            bb += 48;
                            st = Convert.ToString((char)(bb));
                            WriteCommASCII(st);
                            i += 4;
                        }
                        j--;
                    }
                }
                st = "|}";
                WriteCommASCII(st);
                fnt.Dispose();
                bmp.Dispose();
                img_frm.Dispose();
            }
        }

        //이미지 인쇄(x위치, y위치, 회전(0도=>0, 90도=>1, 180도=>2, 270도=>3), 이미지화일명)
        public void PutImage(int x, int y, int rot, string fname)
        {
            int aHeight;
            int aWidth;
            int i, j, k;
            Byte bb;
            string st;

            if (File.Exists(fname) == false)
            {
                return;
            }
            using (Bitmap bmp = new Bitmap(fname))
            {

                if ((rot == 0) || (rot == 2))
                {
                    aHeight = bmp.Height;
                    aWidth = ((bmp.Width + 7) >> 3) * 8;
                    st = "{" + "SG;" + String.Format("{0:0000}", x) + "," + String.Format("{0:0000}", y) + "," + String.Format("{0:0000}", aWidth) + "," + String.Format("{0:0000}", aHeight) + ",0,";
                    WriteCommASCII(st);
                }
                else
                {
                    aHeight = ((bmp.Height + 7) >> 3) * 8;
                    aWidth = bmp.Width;
                    st = "{" + "SG;" + String.Format("{0:0000}", x) + "," + String.Format("{0:0000}", y) + "," + String.Format("{0:0000}", aHeight) + "," + String.Format("{0:0000}", aWidth) + ",0,";
                    WriteCommASCII(st);
                }

                if (rot == 0)
                {
                    j = 0;
                    while (j < aHeight)
                    {
                        i = 0;
                        while (i < aWidth)
                        {
                            bb = 0;
                            for (k = 0; k < 4; k++)
                            {
                                if ((i + k) < bmp.Width)
                                {
                                    if (bmp.GetPixel(i + k, j) == Color.FromArgb(255, 0, 0, 0))
                                    {
                                        bb = (byte)(bb + (1 << (3 - k)));
                                    }
                                }
                            }
                            bb += 48;
                            st = Convert.ToString((char)(bb));
                            WriteCommASCII(st);
                            i += 4;
                        }
                        j++;
                    }
                }

                if (rot == 2)
                {
                    j = aHeight - 1;
                    while (j >= 0)
                    {
                        i = aWidth - 1;
                        while (i >= 0)
                        {
                            bb = 0;
                            for (k = 0; k < 4; k++)
                            {
                                if ((i - k) < bmp.Width)
                                {
                                    if (bmp.GetPixel(i - k, j) == Color.FromArgb(255, 0, 0, 0))
                                    {
                                        bb = (byte)(bb + (1 << (3 - k)));
                                    }
                                }
                            }
                            bb += 48;
                            st = Convert.ToString((char)(bb));
                            WriteCommASCII(st);
                            i -= 4;
                        }
                        j--;
                    }
                }

                if (rot == 1)
                {
                    j = 0;
                    while (j < aWidth)
                    {
                        i = aHeight - 1;
                        while (i >= 0)
                        {
                            bb = 0;
                            for (k = 0; k < 4; k++)
                            {
                                if ((i - k) < bmp.Height)
                                {
                                    if (bmp.GetPixel(j, i - k) == Color.FromArgb(255, 0, 0, 0))
                                    {
                                        bb = (byte)(bb + (1 << (3 - k)));
                                    }
                                }
                            }
                            bb += 48;
                            st = Convert.ToString((char)(bb));
                            WriteCommASCII(st);
                            i -= 4;
                        }
                        j++;
                    }
                }

                if (rot == 3)
                {
                    j = aWidth - 1;
                    while (j >= 0)
                    {
                        i = 0;
                        while (i < aHeight)
                        {
                            bb = 0;
                            for (k = 0; k < 4; k++)
                            {
                                if ((i + k) < bmp.Height)
                                {
                                    if (bmp.GetPixel(j, i + k) == Color.FromArgb(255, 0, 0, 0))
                                    {
                                        bb = (byte)(bb + (1 << (3 - k)));
                                    }
                                }
                            }
                            bb += 48;
                            st = Convert.ToString((char)(bb));
                            WriteCommASCII(st);
                            i += 4;
                        }
                        j--;
                    }
                }
                st = "|}";
                WriteCommASCII(st);
                bmp.Dispose();
            }
        }


        private void button1_Click(object sender, EventArgs e)
        {
            Port_Open();
            if (serialPort1.IsOpen)
            {
                listBox1.Items.Add("통신시작!!!");
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            serialPort1.Close();
            listBox1.Items.Add("통신종료!!!");
        }

        // 숫자인지 아닌지를 체크하는 함수
        public bool IsNumber(String strNumber)
        {
            Regex objNotNumberPattern = new Regex("[^0-9.-]");
            Regex objTwoDotPattern = new Regex("[0-9]*[.][0-9]*[.][0-9]*");
            Regex objTwoMinusPattern = new Regex("[0-9]*[-][0-9]*[-][0-9]*");
            String strValidRealPattern = "^([-]|[.]|[-.]|[0-9])[0-9]*[.]*[0-9]+$";
            String strValidIntegerPattern = "^([-]|[0-9])[0-9]*$";
            Regex objNumberPattern = new Regex("(" + strValidRealPattern + ")|(" + strValidIntegerPattern + ")");
            return !objNotNumberPattern.IsMatch(strNumber) && !objTwoDotPattern.IsMatch(strNumber) &&
                  !objTwoMinusPattern.IsMatch(strNumber) && objNumberPattern.IsMatch(strNumber);
        }

        private String GetStatus(String data)
        {
            String result = "상태형식 잘못됨!";
            String tt = data.Substring(0, 2);
            try{
                if(IsNumber(tt)){
                    int stat= int.Parse(tt);
                    switch(stat){
				      case 0: result = "정상동작!"; break;
				      case 1: result = "헤드열림!"; break;
				      case 2: result = "동작중..."; break;
				      case 3: result = "발행중..."; break;
				      case 4: result = "중지상태"; break;
				      case 6: result = "명령에러"; break;
				      case 7: result = "통신에러"; break;
				      case 11: result = "용지걸림!"; break;
				      case 12: result = "커터에러!"; break;
				      case 13: result = "용지없음!"; break;
				      case 14: result = "리본없음!"; break;
				      case 15: result = "헤드열림!"; break;
				      case 17: result = "헤드에러!"; break;
				      case 18: result = "헤드이상과열!"; break;
				      case 21: result = "리본에러!"; break;
				      case 22: result = "리와인더 오버풀!"; break;
				      case 27: result = "정상동작!"; break;
				      case 28: result = "중지상태"; break;
				      case 29: result = "정상동작!"; break; 
				      case 40: result = "발행완료"; break;
				      case 41: result = "피드완료"; break;
				      case 50: result = "플레쉬메모리 다운로드에러"; break;
				      case 51: result = "플레쉬메모리 포맷에러"; break;
				      case 54: result = "플레쉬메로리 용량부족"; break;
				      default: result = "상태알수없음!"; break;
                    }
                }
            }catch {
                result = "상태형식 잘못됨!";
            }
            return result;
        }

        private void Receive_Proc()
        {
            //수신흐름 처리
            int stx_pos, etx_pos;

            stx_pos = recvData.IndexOf("\x2");
            etx_pos = recvData.IndexOf("\x3");

            try
            {
                if ((stx_pos >= 0) && (etx_pos >= 0) && (stx_pos < etx_pos))
                {

                    String st = recvData.Substring(stx_pos + 1, etx_pos - stx_pos - 1);  //정상적인 데이타를 받았을때 처리

                    listBox1.Items.Add(st + "=>" + GetStatus(st));
                    recvData = "";
                }
            }
            catch 
            {
                recvData = "";
            }


        }

        public void UpdateStatus()
        {
            if ((inputData.Trim()) == string.Empty) return;

            if (inputData == "\0")
            {
                listBox1.Items.Add("연결 끊어짐!!!");
                if (serialPort1.IsOpen) serialPort1.Close();
                return;
            }

            recvData += inputData;
            Receive_Proc();
        }

        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            //시리얼 통신에 데이타를 받았을때 이벤트처리
            Byte[] data = new byte[serialPort1.BytesToRead];
            serialPort1.Read(data, 0, data.Length);
            inputData = Encoding.Default.GetString(data, 0, data.Length);
            this.Invoke(new myDelegate(UpdateStatus), new object[] { });
        }

        //사용가능 프린터 종류: B-452
        public string Get452Hangul(string st)
        {
            Encoding oEnc = System.Text.Encoding.GetEncoding(949);
            Byte[] buff = oEnc.GetBytes(st);
            Byte[] oResult = new Byte[2];
            int i, j;

            i = 0;
            j = i;
            while (i <= buff.Length - 1)
            {
                Array.Resize(ref oResult, j + 2);
                if (buff[i] > 128)
                {
                    oResult[j] = buff[i];
                    i = i + 1;
                    oResult[j + 1] = buff[i];
                }
                else
                {
                    oResult[j] = 163;
                    oResult[j + 1] = (byte)(buff[i] + 128);
                }
                i = i + 1;
                j = j + 2;

            }
            return Encoding.Default.GetString(oResult, 0, oResult.Length);
        }

        //사용가능 프린터 종류: B-482/ B-SA4T 시리즈 /B-852/B-SX4T/B-SX5T
        public string GetHangul(string st)
        {
            Encoding oEnc = System.Text.Encoding.GetEncoding(949);
            Byte[] buff = oEnc.GetBytes(st);
            Byte[] oResult = new Byte[2];
            int i, j;

            i = 0;
            j = i;
            while (i <= buff.Length - 1)
            {
                Array.Resize(ref oResult, j + 2);
                if (buff[i] > 128)
                {
                    oResult[j] = buff[i];
                    i = i + 1;
                    oResult[j + 1] = buff[i];
                }
                else
                {
                    oResult[j] = 201;
                    oResult[j + 1] = (byte)(buff[i] + 129);
                }
                i = i + 1;
                j = j + 2;

            }
            return Encoding.Default.GetString(oResult, 0, oResult.Length);
        }


        private void button3_Click(object sender, EventArgs e)
        {
            int print_count = 1;

            if (serialPort1.IsOpen)
            {
                WriteCommASCII("{D0530,1000,0500|}");
                WriteCommASCII("{C|}");
                WriteCommASCII("{PC001;0100,0030,1,1,A,00,B|}");
                WriteCommASCII("{RC001;ABCD|}");
              //  WriteCommASCII("{PC002;0100,0150,1,1,51,00,B|}");
              //  WriteCommASCII("{RC002;" + GetHangul("가나다123ABC DEF") + "|}");
                PutHangul(100, 100, "돋움체", 32, true, 0, "이미지한글테스트인쇄");
                WriteCommASCII("{XS;I," + String.Format("{0:0000}", print_count) + ",0000C4001|}");

            }

        }

        private void button4_Click(object sender, EventArgs e)
        {
            int print_count = 1;

            if (serialPort1.IsOpen)
            {
                WriteCommUTF8("{D2030,1000,2000|}");
                WriteCommUTF8("{C|}");
                WriteCommUTF8("{PC001;0100,0100,1,1,A,00,B|}");
                WriteCommUTF8("{RC001;ABCD|}");
                WriteCommUTF8("{PV01;0100,0150,0040,0040,01,0,00,B|}");
                WriteCommUTF8("{RV01;가나다123abcDEF (주)티비에스|}");
                WriteCommUTF8("{XS;I," + String.Format("{0:0000}", print_count) + ",0002C5001|}");

            }

        }

        private void button5_Click(object sender, EventArgs e)
        {
            if (serialPort1.IsOpen)
            {
                serialPort1.Close();
            }
            Close(); 
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            comboBox1.SelectedIndex = 0;
        }

        private void button6_Click(object sender, EventArgs e)
        {
            WriteCommASCII("{WS|}");
        }

        private void button7_Click(object sender, EventArgs e)
        {
            listBox1.Items.Clear();
        }
    }
}